#include "LaBr3Array.hh"

using namespace std;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
LaBr3Array::LaBr3Array(MaterialList *ptMaterialList,G4LogicalVolume *ptWorld)   {
  cout<<"LaBr3Array Constructor"<<endl;

  fMaterialList        = ptMaterialList;
  lWorld               = ptWorld;
  //fAbsorberThicknessPb = fAbsorberThicknessSn = fAbsorberThicknessAl = 0.0;
  fNumberOfCrystals    = 0;
  
  fHousingThicknessFront  = 0.;
  fHousingThicknessSide   = 0.;

  fInsulationThicknessFront = 0.;
  fInsulationThicknessSide = 0.;

  for(int i=0;i<NUMBEROFLABR3ARRAYCRYSTALS;i++)  {
    fPosX[i] = fPosY[i] = fPosZ[i] = 0.0;
  }
 
  //CreateArrayXYZPsiThetaPhi();
  cout<<"Exiting LaBr3Array Constructor"<<endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
LaBr3Array::~LaBr3Array()  {
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void LaBr3Array::ResetValues()  {
  fCrystalMult = 0;
  for(int i=0;i<fNumberOfCrystals;i++)  {
    fCrystalFlag[i]   = false;
    fCrystalEnergy[i] = 0.0;
    fCrystalTime[i]   = 0.0;
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
float LaBr3Array::GetCrystalMeasuredEnergy(int a)  {
  if(fCrystalEnergy[a]==0) return 0.;
  float dummy;
  if(fTypeOfEnergyResolution==1) dummy = (fEnergyResolution[0] + fEnergyResolution[1]*fCrystalEnergy[a]);
  else if(fTypeOfEnergyResolution==2) dummy = fEnergyResolution[0]*TMath::Power(fCrystalEnergy[a],fEnergyResolution[1]);
  else {
    cout<<"Wrong resolution option for LaBr3Array. Aborting program."<<endl; 
    abort();
  }
  //Observed energy in the detector
  return G4RandGauss::shoot(fCrystalEnergy[a],dummy);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
float LaBr3Array::GetCrystalMeasuredTime(int a)  {
  if(fCrystalTime[a]==0) return 0.;
  //Observed time in the detector
  float dummy = (fTimeResolution[0] - fTimeResolution[1]*TMath::Power(fCrystalEnergy[a],0.5));
  return G4RandGauss::shoot(fCrystalTime[a],dummy);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void LaBr3Array::CreateArrayXYZPsiThetaPhi()  {
  G4cout<<"Creating the LaBr3Array array"<<endl;
  float x,y,z,psi,theta,phi,crystalDiameter,crystalLength;
  x = y = z = psi = theta = phi = crystalDiameter = crystalLength = 0.;
  int id;
  G4RotationMatrix Rot3D, Rot3D2;
  
  fFileIn  = fopen("./geometry/LaBr3Array_geometry_in.txt","r");
  fFileOut = fopen("./geometry/LaBr3Array_geometry_out.txt","w");
  for(int i=0;!feof(fFileIn)&& i<NUMBEROFLABR3ARRAYCRYSTALS;i++)  {
    fscanf(fFileIn,"%i %f %f %f %f %f %f %f %f %s",&id,&x,&y,&z,&psi,&theta,&phi,&crystalDiameter,&crystalLength,fInsulationMaterialName);
    G4cout<<"Read detector "<<i<<": "<<x<<" "<<y<<" "<<z<<" "<<psi<<" "<<theta<<" "<<phi<<endl;
    if(id==-1) break;
    if(id<1) continue;

    sLaBr3ArrayCrystal = new G4Tubs("LaBr3ArrayCrystal",0.0,crystalDiameter*cm/2.0,crystalLength*cm/2.0,0.0*deg,360.0*deg);
    lLaBr3ArrayCrystal = new G4LogicalVolume(sLaBr3ArrayCrystal,fMaterialList->GetMaterial("LaBr3"),"lLaBr3ArrayCrystal",0,0,0);
    
    G4VisAttributes* visAtt = new G4VisAttributes(G4Colour(0.5,0.5,1.0)); 
    lLaBr3ArrayCrystal->SetVisAttributes(visAtt);

    //The Housing
    if(fHousingThicknessFront>0. && fHousingThicknessSide>0.)  {
      sLaBr3ArrayAlHouseOut = new G4Tubs("sLaBr3ArrayAlHouseOut",
                                         0.0,(crystalDiameter+2*fInsulationThicknessSide+2*fHousingThicknessSide)*cm/2.0,
                                         (crystalLength+2*fInsulationThicknessFront+2*fHousingThicknessFront)*cm/2.0,
                                         0*deg,360*deg);
      sLaBr3ArrayAlHouseIn = new G4Tubs("sLaBr3ArrayAlHouseIn",
                                        0.0,(crystalDiameter+2*fInsulationThicknessSide)*cm/2.0,
                                        (crystalLength+2*fInsulationThicknessFront)*cm/2.0,
                                        0*deg,360*deg);
      sLaBr3ArrayAlHouse   = new G4SubtractionSolid("sLaBr3ArrayAlHouse",sLaBr3ArrayAlHouseOut,sLaBr3ArrayAlHouseIn);
      lLaBr3ArrayAlHouse   = new G4LogicalVolume(sLaBr3ArrayAlHouse,fMaterialList->GetMaterial("Al"),"lLaBr3ArrayAlHouse",0,0,0);
      
      // Setting the vis attributes:
      G4VisAttributes* visAttHouse = new G4VisAttributes(G4Colour(1.0,1.0,1.0)); 
      lLaBr3ArrayAlHouse->SetVisAttributes(visAttHouse);
    }
    // The insulation:
    if(fInsulationThicknessFront>0. && fInsulationThicknessSide>0.)  {
      sLaBr3ArrayInsulationOut = new G4Tubs("sLaBr3ArrayInsulationOut",
                                            0.0,(crystalDiameter+2*fInsulationThicknessSide)*cm/2.0,
                                            (crystalLength+2*fInsulationThicknessFront)*cm/2.0,
                                            0*deg,360*deg);

      sLaBr3ArrayInsulationIn = new G4Tubs("sLaBr3ArrayInsulationIn",
                                           0.0,crystalDiameter*cm/2.0,
                                           crystalLength*cm/2.0,
                                           0*deg,360*deg);

      sLaBr3ArrayInsulation   = new G4SubtractionSolid("sLaBr3ArrayInsulation",sLaBr3ArrayInsulationOut,sLaBr3ArrayInsulationIn);
      lLaBr3ArrayInsulation   = new G4LogicalVolume(sLaBr3ArrayInsulation,fMaterialList->GetMaterial(fInsulationMaterialName),
                                                    "InsulationMaterial",0,0,0);
      
      // Setting the vis attributes:
      G4VisAttributes* visAttInsulation = new G4VisAttributes(G4Colour(1.0,1.0,.0)); 
      lLaBr3ArrayInsulation->SetVisAttributes(visAttInsulation);
    }

    fNumberOfCrystals++;

    //The first digit is for the array type, the second for the different materials inserted
    //The last digit is for the different detector numbers
    id       =  600000 + id -1;
 
    G4ThreeVector Pos(x*cm,y*cm,z*cm); //Position of the center

    Rot3D.set(0, 0, 0);
    Rot3D.rotateX(psi*degree);
    Rot3D.rotateY(theta*degree);  
    Rot3D.rotateZ(phi*degree);  

    sprintf(fTempname,"pLaBr3ArrayCrystal[%i]",i);

    pLaBr3ArrayCrystal[i] = new G4PVPlacement(G4Transform3D(Rot3D,Pos),
                                              lLaBr3ArrayCrystal,fTempname,lWorld,false,id);

    // Putting the housing
    if(fHousingThicknessSide>0. && fHousingThicknessFront>0.)  {
      sprintf(fTempname,"pLaBr3ArrayAlHouse[%i]",i);

      pLaBr3ArrayAlHouse[i] = new G4PVPlacement(G4Transform3D(Rot3D,Pos),
                                                lLaBr3ArrayAlHouse,fTempname,lWorld,false,id+10000);
    }
    
    // Putting the insulationr:
    if(fInsulationThicknessSide>0. && fInsulationThicknessFront>0.)  {
      sprintf(fTempname,"pLaBr3ArrayInsulation[%i]",i);
      
      pLaBr3ArrayInsulation[i] = new G4PVPlacement(G4Transform3D(Rot3D,Pos),
                                                   lLaBr3ArrayInsulation,fTempname,lWorld,false,id+20000);
    }

    // Assigning the positions:
    fPosX[i] = Pos.getX();
    fPosY[i] = Pos.getY();
    fPosZ[i] = Pos.getZ();
    //-----------------------------------------
    //------------------------------------------------
    cout << id << " " << fPosX[i]/cm 
         << " " << fPosY[i]/cm 
         << " " << fPosZ[i]/cm << " " << endl;
    //----------------------------------------------------
    float distance = sqrt(fPosX[i]/mm*fPosX[i]/mm
                          + fPosY[i]/mm*fPosY[i]/mm	 
                          + fPosZ[i]/mm*fPosZ[i]/mm);

    float thetaPlaced = acos(fPosZ[i]/mm/distance);
    float phiPlaced = acos(fPosX[i]/mm/distance/sin(thetaPlaced));

    if(fPosY[i]/mm < 0.0) phiPlaced = 2.0*3.14159-phiPlaced;
    thetaPlaced = thetaPlaced/3.14159*180.0;
    phiPlaced   = phiPlaced/3.14159*180.0;

    if(abs(fPosY[i]/mm)<1.0 && fPosX[i]/mm>=0.0) phiPlaced = 0.0;
    if(abs(fPosY[i]/mm)<1.0 && fPosX[i]/mm<0.0)  phiPlaced = 180.0;

    fprintf(fFileOut,"%i %f %f %f \n", i, thetaPlaced, phiPlaced, distance);
  }
  fclose(fFileIn);
  fclose(fFileOut);
}
